@@ -16,16 +16,16 @@ try:
                 [(1, '_', u'Page arg 1', []),
                  (1, '_', u'Page arg 2', []),
                  (10, 'gettext', u'Begin', []),
-                 (14, '_', u'Hi there!', [u'Hi there!']),
+                 (14, '_', u'Hi there!', [u'TRANSLATOR: Hi there!']),
                  (19, '_', u'Hello', []),
                  (22, '_', u'Welcome', []),
                  (25, '_', u'Yo', []),
-                 (36, '_', u'The', [u'Ensure so and', u'so, thanks']),
+                 (36, '_', u'The', [u'TRANSLATOR: Ensure so and', u'so, thanks']),
                  (36, 'ungettext', (u'bunny', u'bunnies', None), []),
-                 (41, '_', u'Goodbye', [u'Good bye']),
+                 (41, '_', u'Goodbye', [u'TRANSLATOR: Good bye']),
                  (44, '_', u'Babel', []),
                  (45, 'ungettext', (u'hella', u'hellas', None), []),
-                 (62, '_', u'Goodbye, really!', [u'HTML comment']),
+                 (62, '_', u'Goodbye, really!', [u'TRANSLATOR: HTML comment']),
                  (65, '_', u'P.S. byebye', []),
                  (71, '_', u'Top', [])]
             self.assertEqual(expected, messages)
