@@ -19,6 +19,7 @@ class MockCache(object):
         self.key = key
         self.kwargs = kwargs.copy()
         self.kwargs.pop('createfunc', None)
+        self.kwargs.pop('defname', None)
         return self.realcache.get(key, **kwargs)
     
 class CacheTest(unittest.TestCase):
@@ -48,6 +49,22 @@ class CacheTest(unittest.TestCase):
         ]
         assert m.kwargs == {}
 
+    def test_cache_enable(self):
+        t = Template("""
+            <%!
+                callcount = [0]
+            %>
+            <%def name="foo()" cached="True">
+                <% callcount[0] += 1 %>
+            </%def>
+            ${foo()}
+            ${foo()}
+            callcount: ${callcount}
+        """, cache_enabled=False)
+        m = self._install_mock_cache(t)
+
+        assert t.render().strip() =="callcount: [2]"
+                
     def test_nested_def(self):
         t = Template("""
         <%!
@@ -335,6 +352,51 @@ class CacheTest(unittest.TestCase):
         x2 = t.render(x=2)
         assert x1.strip() == "foo: 1"
         assert x2.strip() == "foo: 2"
+
+    def test_namespace_access(self):
+        t = Template("""
+            <%def name="foo(x)" cached="True">
+                foo: ${x}
+            </%def>
+
+            <%
+                foo(1)
+                foo(2)
+                local.cache.invalidate_def('foo')
+                foo(3)
+                foo(4)
+            %>
+        """)
+        assert result_lines(t.render()) == ['foo: 1', 'foo: 1', 'foo: 3', 'foo: 3']
+        
+    def test_invalidate(self):
+        t = Template("""
+            <%def name="foo()" cached="True">
+                foo: ${x}
+            </%def>
+
+            <%def name="bar()" cached="True" cache_type='dbm' cache_dir='./test_htdocs'>
+                bar: ${x}
+            </%def>
+            ${foo()} ${bar()}
+        """)
+        assert result_lines(t.render(x=1)) == ["foo: 1", "bar: 1"]
+        assert result_lines(t.render(x=2)) == ["foo: 1", "bar: 1"]
+        t.cache.invalidate_def('foo')
+        assert result_lines(t.render(x=3)) == ["foo: 3", "bar: 1"]
+        t.cache.invalidate_def('bar')
+        assert result_lines(t.render(x=4)) == ["foo: 3", "bar: 4"]
+        
+        t = Template("""
+            <%page cached="True" cache_type="dbm" cache_dir="./test_htdocs"/>
+            
+            page: ${x}
+        """)
+        assert result_lines(t.render(x=1)) == ["page: 1"]
+        assert result_lines(t.render(x=2)) == ["page: 1"]
+        t.cache.invalidate_body()
+        assert result_lines(t.render(x=3)) == ["page: 3"]
+        assert result_lines(t.render(x=4)) == ["page: 3"]
         
         
     def _install_mock_cache(self, template):
